import pyedm
from logger import log
from tree_node import TreeNode
import bpy
import animation as anim
from math_tools import IDENTITY_MATRIX

def build_bone_id(armature_name, bone_name):
    return f'{armature_name} : {bone_name}'

class BoneNode(TreeNode):
    def __init__(self, pbone: bpy.types.PoseBone, armature: bpy.types.Armature) -> None:
        super().__init__()
        self.name = build_bone_id(armature.name, pbone.name)
        self.armature = armature
        self.pbone = pbone
        self.bone = pbone.bone # rest bone
        
        self.update_matrices()

        self.edm_nodes = anim.extract_bone_animation(self, armature)

    def update_matrices(self):
        self.mat_inv = (self.bone.matrix_local).inverted()
        if self.pbone.parent:
            self.mat = self.pbone.parent.matrix.inverted() @ self.pbone.matrix
        else:
            self.mat = self.pbone.matrix

    def bootstrap_edm_bone_tree(self, base_edm_node):
        if self.parent:
            self.parent.edm_nodes[1].addChild(self.edm_nodes[0])
        else:
            base_edm_node.addChild(self.edm_nodes[0])


def build_bones_tree(parent, bones_list, armature):
    bones = [BoneNode(b, armature) for b in bones_list]

    for b in bones:
        for c in b.pbone.children:
            for bb in bones:
                if c == bb.pbone:
                    b.add_child(bb)
                    break

    for b in bones:
        b.bootstrap_edm_bone_tree(parent)

    return bones

def export_armature(armature, parent, bones_dict):
    # Pose bones contain pose data for the current frame, while regular bone objects store the default state of the bone.
    pose_bones = armature.pose.bones
    if not pose_bones:
        log.warning(f'Armature {armature.name} is empty')
        return
    

    #root = pyedm.Node(f'Armature {armature.name} Root')
    #root = pyedm.Transform(f'Armature {armature.name} Root', armature.matrix_local)
    #parent = parent.addChild(root)
    
    bone_nodes = build_bones_tree(parent, pose_bones, armature)
    
    bones_dict.update({(x.name, x) for x in bone_nodes})
    
    return parent
